/************************************************************************
  PROG.C      Main du programme de calcul de position des plantes
              principales du systme solaire (Mercure --> Pluton)
              Longitudes des prihlies non calcules
	Par Thierry Graff
		-	mai 1997 :	organisation des calculs  partir de sources de David Coronat.
	D'aprs les calculs de Meeus.
 ************************************************************************/

#include <memory.h>
#include <math.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <dos.h>

typedef struct{
	double x1;
	double x2;
	double x3;
	}coord;

double Jour_julien(char *date, char *heure);
void Calc_M(double T);

void Calc_Soleil(double T);
void Calc_Mercure(double T);
void Calc_Venus(double T);
void Calc_Mars(double T);
void Calc_Jupiter(double T);
void Calc_Saturne(double T);
void Calc_Uranus(double T);
void Calc_Neptune(double T);
void Calc_Pluton(double T);

//void Pol_to_cart(coord *pol, coord *cart);
void HCpol_to_GCpol(double T);

double Kepler_eq(double e, double M);
double Mod360(double nombre);

#define PI 3.141592653589793
#define PIs180 0.0174532925199433      // = PI/180.0

char *nom[]={"Mercure","Venus","Terre/Soleil","Mars",
	     "Jupiter","Saturne","Uranus","Neptune","Pluton"};

// Anomalies moyennes; l'indice correspond  Meus.
double M1,M2,M3,M4,M5,M6,M7,M8;

// Variables utiles au calcul des plantes gazeuses.
//              (up=upsilon, z=zeta)
//              sv = sin(V), s2v = sin(2*V) etc...
double up,P,Q,S,V,W,z,G,H,
       up2,sv,s2v,cv,c2v,sw,sz,s2z,s3z,s4z,s5z,cz,c2z,c3z,c4z,c5z,
       sq,s2q,s3q,s4q,cq,c2q,c3q,c4q,cg,sg,sh,s2h,ch,c2h;

	/*************** COORDONNEES *********************
		HC: Hliocentriques eCliptiques (r,l,b)
		GC: Gocentriques eCliptiques (DELTA,beta,lambda)
		GQ: Gocentriques eQuatoriales (DELTA,alpha,delta)
	0:Mercure, 1:Vnus, 2:Terre / Soleil, 3:Mars, 4:Jupiter,
	5:Saturne, 6:Uranus, 7: Neptune, 8:Pluton.
	L'indice est dcal par rapport aux anomalies moyennes.
	***************************************************/
	coord	HCpol[9],         // HC polaires; x1=r, x2=l, x3=b.
   		HCcart[9],        // HC cartsiennes; x1=X, x2=Y, x3=Z.
			GCpol[9];         // GC polaires; x1=delta, x2=lambda, x3=bta.

void main()
{
	char date_entree[256],heure_entree[256];
  char wait[256];
	double jd,T;
	int i;

	/*printf("Entrez la date (JJ/MM/AAAA): ");
	scanf( "%s", date_entree);
	printf("Entrez l\'heure (HH/MM/SS): ");
	scanf( "%s", heure_entree);
   */
	jd = Jour_julien("27/08/1967","09h05m00s");
	//jd = Jour_julien(date_entree,heure_entree);
	T = (jd-2415020.0)/36525.0;  // nb de siecles depuis 0.5/01/1900
	printf("\nJD = %f\tT = %.9f\n",jd,T);

	Calc_M(T);      //calcule anomalies moyennes.
			// ATTENTION: MANQUE DE RIGUEUR

	Calc_Soleil(T);
	Calc_Mercure(T);
	Calc_Venus(T);
	Calc_Mars(T);

	/*************************************************************
	Variables  pour le calcul des lourdes.
	A part up, toutes les variables sont converties en radians.
	**************************************************************/
	up = T/5.0 + 0.1;
		up2 = up*up;
	P = (237.47555 + 3034.9061*T)*PIs180;
	Q = (265.91650 + 1222.1139*T)*PIs180;
		sq = sin(Q);
		s2q = sin(2.0*Q);
		s3q = sin(3.0*Q);
		s4q = sin(4.0*Q);
		cq = cos(Q);
		c2q = cos(2.0*Q);
		c3q = cos(3.0*Q);
		c4q = cos(4.0*Q);
	S = (243.51721 + 428.4677*T)*PIs180;
	V = 5*Q - 2*P;
		sv = sin(V);
		s2v = sin(2.0*V);
		cv = cos(V);
		c2v = cos(2.0*V);
	W = 2*P - 6*Q + 3*S;
		sw = sin(W);
	z = Q - P;
		sz = sin(z);
		s2z = sin(2.0*z);
		s3z = sin(3.0*z);
		s4z = sin(4.0*z);
		s5z = sin(5.0*z);
		cz = cos(z);
		c2z = cos(2.0*z);
		c3z = cos(3.0*z);
		c4z = cos(4.0*z);
		c5z = cos(5.0*z);
	 G = (83.76922 + 218.4901*T)*PIs180;
		cg = cos(G);
		sg = sin(G);
	 H = 2.0*G - S;
		sh = sin(H);
		s2h = sin(2.0*H);
		ch = cos(H);
		c2h = cos(2.0*H);

	Calc_Jupiter(T);
	Calc_Saturne(T);
	Calc_Uranus(T);
	Calc_Neptune(T);

	// Calcul coord. hlio. cliptiques cartsiennes.
	for(i=0;i<9;i++)
		{
		Pol_to_cart(&HCpol[i],&HCcart[i]);
		}


	// Calcul coord cliptiques gocentriques.
	HCpol_to_GCpol(T);

	// Affichage
	for(i=0;i<9;i++)
		{/*
		printf("\n%s: r = %.3f , l = %.3f , b = %.3f",
		nom[i],HCpol[i].x1,HCpol[i].x2,HCpol[i].x3);
		printf("\n\tX = %.3f , Y = %.3f , Z = %.3f",
		HCcart[i].x1,HCcart[i].x2,HCcart[i].x3);
      */
		printf("\n%s: \tDELTA = %.3f , lambda = %.3f , beta = %.3f",
		nom[i],GCpol[i].x1,GCpol[i].x2,GCpol[i].x3);

		}

   scanf( "%s", wait);

}// fin main


/*************************************************************************
			Jour_julien
	ENTREE: Date de format JJ/MM/AAAA (sparateurs quelconques sauf espace).
		Heure de format HH/MM/SS (sparateurs quelconques sauf espaces).
	RETOUR: Jour julien correspondant  cette date.
**************************************************************************/
double Jour_julien(char *date, char *heure)
{
	char *tampon;
	int j,m,an,h,min,s;
	double jour,y1,m1,a,b,res;

	//printf( "La date est: %s\n",date );
	tampon="\0";
	strncpy(tampon,date,2);
	j = atoi(tampon);
	tampon="\0";
	strncpy(tampon,date+3,2);
	m = atoi(tampon);
	tampon="\0";
	strncpy(tampon,date+6,4);
	an = atoi(tampon);

   /*j=27;
   m=08;
   an=1967;
   */
	printf("Jour: %d\tMois: %d\tAnne: %d\n",j,m,an);
	//printf( "\nL\'heure est: %s\n",heure );
	tampon="\0";
	strncpy(tampon,heure,2);
	h = atoi(tampon);
	tampon="\0";
	strncpy(tampon,heure+3,2);
	min = atoi(tampon);
	tampon="\0";
	strncpy(tampon,heure+6,2);
	s = atoi(tampon);

   /*h=9;
   min=5;
   s=0;
   */
	printf("Heures: %d\tMinutes: %d\tSecondes: %d\n",h,min,s);

	jour = j + h/24.0 + min/1440.0 + s/86400.0;
	//printf("\n jour  = %f",jour);
	if(m>2)
	{
		y1 = an;
		m1 = m;
	}
	else
	{
		y1 = an-1;
		m1 = m+12;
	};
	//printf("\ny1 = %f   m1 = %f",y1,m1);

	res = floor(365.25*y1) + floor(30.6001*(m1+1)) + jour + 1720994.5;

	if((an+m/100.0+jour/10000.0)>=1582.1015)
	{
		a = floor(y1/100.0);
		b = 2-a+floor(a/4.0);
		//printf("\na = %f   b = %f",a,b);
		res = res + b;
	};

	return res;
}//fin Jour_julien


/***************************************************************************
			Calc_M
	Calcule les anomalies moyennes, de Mercure  Neptune
	exprimes en degrs, rduites  [0,360].
	M1  M8 sont des variables globales.
	Attention: M3 est la valeur du Soleil.
	Les indices sont ceux de Meus (diffrent de ceux de HCPol et HCCart).
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_M(double T)
{
	M1 = 102.27938 + (149472.51529 + 7.0E-06*T)*T;
	M2 = 212.60322 + (58517.80387 + 0.001286*T)*T;
	M3 = 358.47583 + (35999.04975 - (0.000150 + 0.0000033*T)*T)*T;
	M4 = 319.51913 + (19139.85475 + 0.000181*T)*T;
	M5 = 225.32833 + (3034.69202 - 0.000722*T)*T;
	M6 = 175.46622 + (1221.55147 - 0.000502*T)*T;
	M7 = 72.648778 + (428.3791132 + 0.0000788*T)*T;
	M8 = 37.73063  + (218.4613396 - 0.00007*T)*T;
	M1 = Mod360(M1);
	M2 = Mod360(M2);
	M3 = Mod360(M3);
	M4 = Mod360(M4);
	M5 = Mod360(M5);
	M6 = Mod360(M6);
	M7 = Mod360(M7);
	M8 = Mod360(M8);

}//Fin Calc_M


/***************************************************************************
			Calc_Soleil
	ENTREE: Sicles juliens couls depuis le 0.5 janvier 1900.
	Calcule HCpol[2] et  GCpol[2] (coord gomtriques, pas apparentes)
	RETOUR: aucun.
**************************************************************************/
void Calc_Soleil(double T)
{
	double rsol, lsol;
	double L,M3_rad,ecc,centre;
	double a,b,c,d,e,h;     // variables pour meilleure prcision

	L = 279.6966778 + (36000.768925 + 0.0003025*T)*T;
	L = Mod360(L);
	M3_rad = M3*PIs180;
	ecc = 0.01675104 - (0.0000418 + 0.000000126*T)*T;
	//printf("\nL = %f\nM3 = %f\necc = %f",L,M3,ecc);

	centre = (1.919460 + (-0.004789 - 0.000014*T)*T)*sin(M3_rad)
	       + (0.020094 - 0.000100*T)*sin(2*M3_rad)
	       +  0.000293*sin(3*M3_rad);

	lsol = L + centre;
	rsol = 1.0000002*(1-ecc*ecc)/(1+ecc*cos(M3_rad+centre*PIs180));

	//************ Meilleure prcision ********************
	a  = (153.23 + 22518.7541 * T ) * PIs180;
	b  = (216.57 + 45037.5082 * T ) * PIs180;
	c  = (312.69 + 32964.3577 * T ) * PIs180;
	d  = (350.74 + (445267.1142  - 0.00144*T)*T) * PI / 180.0;
	e  = (231.19 + 20.2 * T ) * PIs180;
	h =  (353.40 + 65928.7155*T) * PIs180;

	rsol = rsol + 0.00000543*sin(a) + 0.00001575*sin(b)
		      + 0.00001627*sin(c) + 0.00003076*cos(d)
		      + 0.00000927*sin(h);


	lsol = lsol  + 0.00134 * cos(a ) + 0.00154 * cos(b )
		     + 0.00200 * cos(c ) + 0.00179 * sin(d )
		     + 0.00178 * sin(e );

		/******* Corrections appores par David (pas dans Meus).
		- 0.00569 - 0.00479 * sin(nl )
		+ (-(17.2327 + 0.01737 * t ) * sin(nl )
		- (1.2729 + 0.00013 * t ) * sin(2 * l ) + 0.2088 * sin(2 * nl )
		- 0.2037 * sin(2 * l1 ) + (0.1261 - 0.00031 * t ) * sin(m ) + 0.0675 * sin(mm )
		- (0.0497 - 0.00012 * t ) * sin(2 * l  + m ) - 0.0342 * sin(2 * l1  - nl )
		- 0.0261 * sin(2 * l1  + mm ) + 0.0214 * sin(2 * l  - m )
		- 0.0149 * sin(2 * l  - 2 * l1  + mm ) + 0.0124 * sin(2 * l  - nl )
		+ 0.0114 * sin(2 * l1  - mm ))/3600.0;
		****************************************************/

	HCpol[2].x1 = rsol;
	HCpol[2].x2 = Mod360(lsol-180);
	HCpol[2].x3 = 0.0;

	GCpol[2].x1 = rsol;
	GCpol[2].x2 = lsol;
	GCpol[2].x3 = 0.0;

}//fin Calc_Soleil


/***************************************************************************
			Calc_Mercure
	Calcule les lments d'orbite de Mercure
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: Sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Mercure(double T)
{
	double LM,a,ecc,i,om,r,
		E,nu,u,l,b;

	LM = 178.179078 + (149474.07078 + 0.0003011*T)*T;
	LM = Mod360(LM);
	a = 0.3870986;
	ecc = 0.20561421 + (0.00002046 - 3.0E-08*T)*T;
	i = 7.002881 + (0.0018608 - 0.0000183*T)*T;
	i = Mod360(i);
	om = 47.145944 + (1.1852083 + 0.0001739*T)*T;
	om = Mod360(om);

	E = Kepler_eq(ecc,M1*PIs180); //E en radians

	nu = 2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2)))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E))
		+7.525E-06*cos((2*M5-M1+53.013)*PIs180)
		+6.802E-06*cos((5*M2-3*M1-259.918)*PIs180)
		+5.457E-06*cos((2*M2-2*M1-71.188)*PIs180)
		+3.569E-06*cos((5*M2-M1-77.75)*PIs180);

	u = LM + nu - M1 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = l + 0.00204*cos((5*M2-2*M1+12.220)*PIs180)
	      + 0.00103*cos((2*M2-M1-160.692)*PIs180)
	      + 0.00091*cos((2*M5-M1-37.003)*PIs180)
	      + 0.00078*cos((5*M2-3*M1+10.137)*PIs180);
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;

	//printf("\nMercure:LM = %f \tecc = %f \ti = %f",LM,ecc,i);
	//printf("\nom = %f \tM1 = %f \tE = %f \tnu = %f",om,M1,E*180.0/PI,nu);
	//printf("\nMercure : \tr = %f \tu = %f \tl = %f \tb = %f ",r,u,l,b);
	HCpol[0].x1=r;
	HCpol[0].x2=l;
	HCpol[0].x3=b;
}//Fin Calc_Mercure


/***************************************************************************
			Calc_Venus
	Calcule les lments d'orbite de Venus
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Venus(double T)
{
	double LM,a,ecc,i,om,r,
		E,nu,u,l,b;

	LM = 342.767053 + (58519.21191 + 0.0003097*T)*T
	    + 0.00077*sin((237.24+150.27*T)*PIs180);
	LM = Mod360(LM);
	a = 0.7233316;
	ecc = 0.00682069 + (-0.00004774 + 9.1E-08*T)*T;
	i = 3.393631 + (0.0010058 - 1.0E-06*T)*T;
	om = 75.779647 + (0.89985 + 0.00041*T)*T;

	M2 = M2 + 0.00077*sin((237.24+150.27*T)*PIs180);

	E = Kepler_eq(ecc,M2*PIs180); // E en radians

	nu = (2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2))))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E))
		+ 0.000022501*cos((2*M3-2*M2-58.208)*PIs180)
		+ 0.000019045*cos((3*M3-3*M2+92.577)*PIs180)
		+ 6.887E-06*cos((M5-M2-118.09)*PIs180)
		+ 5.172E-06*cos((M3-M2-29.11)*PIs180)
		+ 3.62E-06*cos((5*M3-4*M2-104.208)*PIs180)
		+ 3.283E-06*cos((4*M3-4*M2+63.513)*PIs180)
		+ 3.074E-06*cos((2*M5-2*M2-55.167)*PIs180);

	u = LM + nu - M2 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI+om;
	l = l + 0.00313*cos((2*M3-2*M2-148.225)*PIs180)
		+0.00198*cos((3*M3-3*M2+2.565)*PIs180)
		+0.00136*cos((M3-M2-119.107)*PIs180)
		+0.00096*cos((3*M3-2*M2-135.912)*PIs180)
		+0.00082*cos((M5-M2-208.087)*PIs180);
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;

	HCpol[1].x1=r;
	HCpol[1].x2=l;
	HCpol[1].x3=b;
}//Fin Calc_Venus


/***************************************************************************
			Calc_Mars
	Calcule les lments d'orbite de Mars
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Mars(double T)
{
	double LM,a,ecc,i,om,r,
		E,nu,u,l,b;

	LM = 293.737334 + (19141.69551 + 0.0003107*T)*T
		- 0.01133*sin((3*M5-8*M4+4*M3)*PIs180)
		- 0.00933*cos((3*M5-8*M4+4*M3)*PIs180);
	LM = Mod360(LM);
	a = 1.5236883;
	ecc = 0.09331290 + (0.000092064 - 7.7E-08*T)*T;
	i = 1.850333 + (-0.0006750 + 0.0000126*T)*T;
	om = 48.786442+ (0.7709917 +(-1.4E-06 - 5.33E-06*T)*T)*T;

	M4 = M4 - 0.01133*sin((3*M5-8*M4+4*M3)*PIs180)
		- 0.00933*cos((3*M5-8*M4+4*M3)*PIs180);

	E = Kepler_eq(ecc,M4*PIs180); // E en radians.

	nu = (2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2))))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E))
		+0.000053227*cos((M5-M4+41.1306)*PIs180)
		+0.000050989*cos((2*M5-2*M4-101.9847)*PIs180)
		+0.000038278*cos((2*M5-M4-98.3292)*PIs180)
		+0.000015996*cos((M3-M4-55.555)*PIs180)
		+0.000014764*cos((2*M3-3*M4+68.622)*PIs180)
		+8.966E-06*cos((M5-2*M4+43.615)*PIs180)
		+7.914E-06*cos((3*M5-2*M4-139.737)*PIs180)
		+7.004E-06*cos((2*M5-3*M4-102.888)*PIs180)
		+6.62E-06*cos((M3-2*M4+113.202)*PIs180)
		+4.93E-06*cos((3*M5-3*M4-76.243)*PIs180)
		+4.693E-06*cos((3*M3-5*M4+190.603)*PIs180)
		+4.571E-06*cos((2*M3-4*M4+244.702)*PIs180)
		+4.409E-06*cos((3*M5-M4-115.828)*PIs180);

	u = LM + nu - M4 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = l   +0.00705*cos((M5-M4-48.958)*PIs180)
		+0.00607*cos((2*M5-M4-188.35)*PIs180)
		+0.00445*cos((2*M5-2*M4-191.897)*PIs180)
		+0.00388*cos((M3-2*M4+20.495)*PIs180)
		+0.00238*cos((M3-M4+35.097)*PIs180)
		+0.00204*cos((2*M3-3*M4+158.638)*PIs180)
		+0.00177*cos((3*M4-M2-57.602)*PIs180)
		+0.00136*cos((2*M3-4*M4+154.093)*PIs180)
		+0.00104*cos((M5+17.618)*PIs180);
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;

	HCpol[3].x1=r;
	HCpol[3].x2=l;
	HCpol[3].x3=b;
}//Fin Calc_Mars


/***************************************************************************
			Calc_Jupiter
	Calcule les lments d'orbite de Jupiter
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Jupiter(double T)
{
	double LM,a,ecc,i,om,r,
	       E,nu,u,l,b,
	       AA,BB,ecc1;


	LM = 238.049257 + (3036.301986 + (0.0003347 - 0.00000165*T)*T)*T;
	AA = (0.331364-0.010281*up-0.004692*up2)*sv
	+ (0.003228-0.064436*up+0.002075*up2)*cv
	- (0.003083+0.000275*up-0.000489*up2)*s2v
	+ 0.002472*sw
	+ 0.013619*sz
	+ 0.018472*s2z
	+ 0.006717*s3z
	+ 0.002775*s4z
	+ (0.007275-0.001253*up)*sz*sq
	+ 0.006417*s2z*sq
	+ 0.002439*s3z*sq
	- (0.033839+0.001125*up)*cz*sq
	- 0.003767*c2z*sq
	- (0.035681+0.001208*up)*sz*sq
	- 0.004261*s2z*cq
	+ 0.002178*cq
	+ (-0.006333+0.001161*up)*cz*cq
	- 0.006675*c2z*cq
	- 0.002664*c3z*cq
	- 0.002572*sz*s2q
	- 0.003567*s2z*s2q
	+ 0.002094*cz*c2q
	+ 0.003342*c2z*c2q;
	LM = LM + AA;
	LM = Mod360(LM);

	a = 5.202561
	+ (-263.0*cv
	+ 205.0*cz
	+ 693.0*c2z
	+ 312.0*c3z
	+ 147.0*c4z
	+ 299.0*sz*sq
	+ 181.0*c2z*sq
	+ 204.0*s2z*cq
	+ 111.0*s3z*cq
	- 337.0*cz*cq
	- 111.0*c2z*cq)*1.0E-06;

	ecc = 0.04833475 + (0.000164180 +(-4.676E-07 - 1.7E-09*T)*T)*T;
	ecc1 = (3606.0 + 130.0*up - 43.0*up2)*sv
	+ (1289.0 - 580.0*up)*cv
	- 6764.0*sz*sq
	- 1110.0*s2z*sq
	- 224.0*s3z*sq
	- 204.0*sq
	+ (1284.0 + 116.0*up)*cz*sq
	+ 188.0*c2z*sq
	+ (1460.0 + 130.0*up)*sz*cq
	+ 224.0*s2z*cq
	- 817.0*cq
	+ 6074.0*cz*cq
	+ 992.0*c2z*cq
	+ 508.0*c3z*cq
	+ 230.0*c4z*cq
	+ 108.0*c5z*cq
	- (956.0 + 73.0*up)*sz*s2q
	+ 448.0*s2z*s2q
	+ 137.0*s3z*s2q
	+ (-997.0 + 108.0*up)*cz*s2q
	+ 480.0*c2z*s2q
	+ 148.0*c3z*s2q
	+ (-956.0 + 99.0*up)*sz*c2q
	+ 490.0*s2z*c2q
	+ 158.0*s3z*c2q
	+ 179.0*c2q
	+ (1024.0 + 75.0*up)*cz*c2q
	- 437.0*c2z*c2q
	- 132.0*c3z*c2q;
	ecc1 = ecc1*1.0E-07;

	i = 1.308736 + (-0.0056961 + 0.0000039*T)*T;
	i = Mod360(i);
	om = 99.443414 + (1.01053 + (0.00035222 - 8.51E-06*T)*T)*T;
	om = Mod360(om);

	BB = (0.007192-0.003147*up)*sv
	+ (-0.020428-0.000675*up+0.000197*up2)*cv
	+ (0.007269+0.000672*up)*sz*sq
	- 0.004344*sq
	+ 0.034036*cz*sq
	+ 0.005614*c2z*sq
	+ 0.002964*c3z*sq
	+ 0.037761*sz*cq
	+ 0.006158*s2z*cq
	- 0.006603*cz*cq
	- 0.005356*sz*s2q
	+ 0.002722*s2z*s2q
	+ 0.004483*cz*s2q
	- 0.002642*c2z*s2q
	+ 0.004403*sz*c2q
	- 0.002536*s2z*c2q
	+ 0.005547*cz*c2q
	- 0.002689*c2z*c2q;

	M5 = M5 + AA - (BB/ecc);
	ecc = ecc + ecc1;

	E = Kepler_eq(ecc,M5*PIs180); //E en radians

	nu = 2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2)))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E));

	u = LM + nu - M5 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;

	HCpol[4].x1=r;
	HCpol[4].x2=l;
	HCpol[4].x3=b;

}//Fin Calc_Jupiter


/***************************************************************************
			Calc_Saturne
	Calcule les lments d'orbite de Saturne
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Saturne(double T)
{
	double LM,a,ecc,i,om,r,
	       E,nu,u,l,b,
	       AA,BB,ecc1,
	       k,s2k,s3k,s4k,c2k,c3k,c4k; //k = psi.

	k = S - Q;
	c2k = cos(2.0*k);
	c3k = cos(3.0*k);
	c4k = cos(4.0*k);
	s2k = sin(2.0*k);
	s3k = sin(3.0*k);
	s4k = sin(4.0*k);

	LM = 266.564377 + (1223.509884 + (0.0003245 -5.8E-06*T)*T)*T;

	AA = (-0.814181+0.018150*up+0.016714*up2)*sv
	+ (-0.010497+0.160906*up-0.004100*up2)*cv
	+ 0.007581*s2v
	- 0.007986*sw
	- 0.148811*sz
	- 0.040786*s2z
	- 0.015208*s3z
	- 0.006339*s4z
	- 0.006244*sq
	+ (0.008931-0.002728*up)*sz*sq
	- 0.016500*s2z*sq
	- 0.005775*s3z*sq
	+ (0.081344+0.003206*up)*cz*sq
	+ 0.015019*c2z*sq
	+ (0.085581+0.002494*up)*sz*cq
	+ (0.025328-0.003117*up)*cz*cq
	+ 0.014394*c2z*cq
	+ 0.006319*c3z*cq
	+ 0.006369*sz*s2q
	+ 0.009156*s2z*s2q
	+ 0.007525*s3k*s2q
	- 0.005236*cz*c2q
	- 0.007736*c2z*c2q
	- 0.007528*c3k*c2q;

	LM = LM + AA;
	LM = Mod360(LM);

	a = 9.554747
	+ (572*up*sv
	+ 2933*cv
	+ 33629L*cz
	- 3081*c2z
	- 1423*c3z
	- 671*c4z
	- 320*c5z
	+ 1098*sq
	- 2812*sz*sq
	+ 688*s2z*sq
	- 393*s3z*sq
	- 228*s4z*sq
	+ 2138*cz*sq
	- 999*c2z*sq
	- 642*c3z*sq
	- 325*c4z*sq
	- 890*cq
	+ 2206*sz*cq
	- 1590*s2z*cq
	- 647*s3z*cq
	- 344*s4z*cq
	+ 2885*cz*cq
	+ (2172+102*up)*c2z*cq
	+ 296*c3z*cq
	- 267*s2z*s2q
	- 778*cz*s2q
	+ 495*c2z*s2q
	+ 250*c3z*s2q
	- 856*sz*c2q
	+ 441*s2z*c2q
	+ 296*c2z*c2q
	+ 211*c3z*c2q
	- 427*sz*s3q
	+ 398*s3z*s3q
	+ 344*cz*c3q
	- 427*c3z*c3q)*1.0E-06;

	ecc = 0.05589232+(-0.0003455+(-7.28E-07 + 7.4E-10*T)*T)*T;

	ecc1 = (-7927+2548*up+91*up2)*sv
	+ (13381+1226*up-253*up2)*cv
	+ (248-121*up)*s2v
	- (305+91*up)*c2v
	+ 412*s2z
	+ 12415*sq
	+ (390-617*up)*sz*sq
	+ (165-204*up)*s2z*sq
	+ 26599*cz*sq
	- 4687*c2z*sq
	- 1870*c3z*sq
	- 821*c4z*sq
	- 377*c5z*sq
	+ 497*c2k*sq
	+ (163-611*up)*cq
	- 12696*sz*cq
	- 4200*s2z*cq
	- 1503*s3z*cq
	- 619*s4z*cq
	- 268*s5z*cq
	- (282+1306*up)*cz*cq
	+ (-86+230*up)*c2z*cq
	+ 461*s2k*cq
	- 350*s2q
	+ (2211-286*up)*sz*cq;
	ecc1 = ecc1 - 2208*s2z*s2q
	- 568*s3z*s2q
	- 346*s4z*s2q
	- (2780+222*up)*cz*s2q
	+ (2022+263*up)*c2z*s2q
	+ 248*c3z*s2q
	+ 242*s3k*s2q
	+ 467*c3k*s2q
	- 490*c2q
	- (2842+279*up)*sz*c2q
	+ (128+226*up)*s2z*c2q
	+ 224*s3z*c2q
	+ (-1594+282*up)*cz*c2q
	+ (2162-207*up)*c2z*c2q
	+ 561*c3z*c2q
	+ 343*c4z*c2q
	+ 469*s3k*c2q
	- 242*c3k*c2q
	- 205*sz*s3q
	+ 262*s3z*s3q
	+ 208*cz*c3q
	- 271*c3z*c3q
	- 382*c3z*s4q
	- 376*s3z*c4q;
	ecc1 = ecc1*1.0E-07;

	om = 112.790414 + (0.8731951 +(-0.00015218 - 5.31E-06*T)*T)*T;
	i = 2.492519 +(-0.0039189 +(-0.00001549 + 4.0E-08*T)*T)*T;

	BB = (0.077108 + 0.007186*up - 0.001533*up2)*sv
	+ (0.045803 - 0.014766*up - 0.000536*up2)*cv
	- 0.007075*sz
	- 0.075825*sz*sq
	- 0.024839*s2z*sq
	- 0.008631*s3z*sq
	- 0.072586*cq
	- 0.150383*cz*cq
	+ 0.026897*c2z*cq
	+ 0.010053*c3z*cq
	- (0.013597 + 0.001719*up)*sz*s2q
	+ (-0.007742 + 0.001517*up)*cz*s2q
	+ (0.013586 - 0.001375*up)*c2z*s2q
	+ (-0.013667 + 0.001239*up)*sz*c2q
	+ 0.011981*s2z*c2q
	+ (0.014861 + 0.001136*up)*cz*c2q
	- (0.013064 + 0.001628*up)*c2z*c2q;

	M6 = M6 + AA - (BB/ecc);
	ecc = ecc + ecc1;

	E = Kepler_eq(ecc,M6*PIs180); //E en radians

	nu = 2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2)))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E));

	u = LM + nu - M6 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;
	b = b + 0.000747*cz*sq
	      + 0.001069*cz*cq
	      + 0.002108*s2z*s2q
	      + 0.001261*c2z*s2q
	      + 0.001236*s2z*c2q
	      - 0.002075*c2z*c2q;
	b = Mod360(b);

	HCpol[5].x1=r;
	HCpol[5].x2=l;
	HCpol[5].x3=b;

}//Fin Calc_Saturne


/***************************************************************************
			Calc_Uranus
	Calcule les lments d'orbite de Uranus
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Uranus(double T)
{
	double LM,a,ecc,i,om,r,
	       E,nu,u,l,b,
	       AA,BB,ecc1,
	       TO,ET,TE;    // TO=tau, ET=eta, TE=teta.

	TO = S - P;
	ET = S - Q;
	TE = G - S;

	LM = 244.19747 + (429.863546+ (0.0003160 - 6.0E-07*T)*T)*T;
	AA = (0.864319-0.001583*up)*sh
		+(0.082222-0.006833*up)*ch
		+0.036017*s2h
		-0.003019*c2h
		+0.008122*sw;
	LM = LM + AA;
	LM = Mod360(LM);

	BB = 0.120303*sh
	+ (0.019472-0.000947*up)*ch
	+ 0.006197*s2h;

	a = 19.21814 - 0.003825*ch;

	ecc = 0.0463444 +(-0.00002658 + 7.7E-08*T)*T;
	ecc1 = ((-3349+163*up)*sh
	+ 20981*ch
	+ 1311*c2h)*1.0E-07;

	om = 73.477111 + (0.4986678 + 0.0013117*T)*T;
	i = 0.772464 + (0.0006253 + 0.0000395*T)*T;

	M7 = M7 + AA - (BB/ecc);
	ecc = ecc + ecc1;

	E = Kepler_eq(ecc,M7*PIs180); //E en radians

	nu = 2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2)))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E)) +
	(
	- 25948.0 + (5795*cos(S)-1165*sin(S)+1388*cos(2.0*S))*sin(ET)
	+ 4985*cz+(1351*cos(S)+5702*sin(S)+1388*sin(2.0*S))*cos(ET)
	- 1230*cos(S)+904*cos(2.0*TE)
	+ 3354*cos(ET)+894*(cos(TE)-cos(3.0*TE))
	)*1.0E-06;

	u = LM + nu - M7 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = l + (0.010122-0.000988*up)*sin(S+ET)
	+ (-0.038581+0.002031*up-0.001910*up2)*cos(S+ET)
	+ (0.034964-0.001038*up+0.000868*up2)*cos(2.0*S+ET)
	+ 0.005594*sin(S+3.0*TE)
	- 0.014808*sz
	- 0.005794*sin(ET)
	+ 0.002347*cos(ET)
	+ 0.009872*sin(TE)
	+ 0.008803*sin(2.0*TE)
	- 0.004308*sin(3.0*TE);
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;
	b = b + (0.000458*sin(ET)-0.000642*cos(ET)-0.000517*cos(4.0*TE))*sin(S)
		-(0.000347*sin(ET)+0.000853*cos(ET)+0.000517*cos(4.0*ET))*cos(S)
		+0.000403*(cos(2.0*TE)*sin(2.0*S)+sin(2.0*TE)*cos(2.0*S));
	b = Mod360(b);

	HCpol[6].x1=r;
	HCpol[6].x2=l;
	HCpol[6].x3=b;

}//Fin Calc_Uranus


/***************************************************************************
			Calc_Neptune
	Calcule les lments d'orbite de Neptune
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Neptune(double T)
{
	double LM,a,ecc,i,om,r,
	       E,nu,u,l,b,
	       AA,BB,ecc1,
	       TO,ET,TE;    // TO=tau, ET=eta, TE=teta.

	TO = G - P;
	ET = G - Q;
	TE = G - S;

	LM = 84.457994 + (219.885914 + (0.0003205 - 6.0E-07*T)*T)*T;
	AA = (-0.589833+0.001089*up)*sh
	+ (-0.056094+0.004658*up)*ch
	- 0.024286*s2h;
	LM = LM + AA;
	LM = Mod360(LM);

	BB = 0.024039*sh
	- 0.025303*ch
	+ 0.006206*s2h
	- 0.005992*c2h;

	a = 30.10957+(-817*sh+8189*ch+781*c2h)*1.0E-06;

	ecc = 0.00899704 + (6.33E-06 - 2.0E-09*T)*T;
	ecc1 = (4389*sh
	+ 4262*ch
	+ 1129*s2h
	+ 1089*c2h)*1.0E-07;

	om = 130.681389 + (1.098935 + (0.00024987 - 4.718E-06*T)*T)*T;

	i = 1.779242 + (-0.0095436 - 9.1E-06*T)*T;

	M8 = M8 + AA - (BB/ecc);
	ecc = ecc + ecc1;

	E = Kepler_eq(ecc,M8*PIs180); //E en radians

	nu = 2*atan(sqrt((1+ecc)/(1-ecc))*(tan(E/2)))*180.0/PI;
	nu = Mod360(nu);

	r = a*(1-ecc*cos(E)) +
	( 40596L
	+ 4992*cz
	+ 2744*cos(ET)
	+ 2044*cos(TE)
	+ 1051*cos(2*TE))*1.0E-06;

	u = LM + nu - M8 - om;
	u = Mod360(u);

	l = atan2(cos(i*PIs180)*sin(u*PIs180),cos(u*PIs180))*180.0/PI + om;
	l = l - 0.009556*sz
	- 0.005178*sin(ET)
	+ 0.002572*sin(2*TE)
	- 0.002972*cos(2*TE)*sg
	- 0.002833*sin(2*TE)*cg;
	l = Mod360(l);

	b = asin(sin(u*PIs180)*sin(i*PIs180))*180.0/PI;
	b = b + 0.000336*cos(2*TE)*sg
	      + 0.000364*sin(2*TE)*cg;
	b = Mod360(b);

	HCpol[7].x1=r;
	HCpol[7].x2=l;
	HCpol[7].x3=b;

}//Fin Calc_Neptune


/***************************************************************************
			Calc_Pluton
	Calcule les lments d'orbite de Pluton
	ainsi que les coordonnes ecliptiques hliocentriques polaires.
	ENTREE: sicles juliens couls depuis le 0.5 janvier 1900.
	RETOUR: aucun.
**************************************************************************/
void Calc_Pluton(double T)
{
		double n_p,l0,lp,p0,pp,o0,op,e,i1,a,p,l1,m,u,o,c,
			rs,xp,yp,l,ls,bs,xs,ys,v,d,
			r,b,zp;
	//      double b1,lx,px,di,r,zp;

	xs = GCpol[2].x1*cos(GCpol[2].x2*PIs180);
	ys = GCpol[2].x1*sin(GCpol[2].x2*PIs180);
	n_p = 36525L*T - 364.5;
//      di=8.2;
	l0 = 1.6406;
	lp = 701214E-10;
	p0 = 3.8978;
	pp = 6.672E-7;
	o0 = 1.9034;
	op = 66.72E-08;
	e = 0.250236;
	i1 = 0.29968;
	a = 39.438712;
	p = p0+pp*n_p;
//      px = p*180.0/M_PI;
	l1 = l0+lp*n_p;
//      lx = l1*180.0/M_PI;
	m = l1-p;
	u = Kepler_eq(e,m);
	v = 2*atan(sqrt((1+e)/(1-e))*(tan(u/2)));
	o = o0 + op*n_p;
	c = v + p - o;
	if(cos(c)==0) d=c;
	else d=atan2(sin(c)*cos(i1),cos(c));
	ls=d+o;
	bs=atan2(sin(d)*sin(i1),cos(i1));
	rs=a*(1-e*cos(u));
	xp=rs*cos(bs)*cos(ls)+xs;
	yp=rs*cos(bs)*sin(ls)+ys;
	zp=rs*sin(bs);
	r=sqrt(xp*xp+yp*yp);
	b=atan2(zp,r)*180.0/PI;
	l=atan2(yp,xp)*180.0/PI;

	HCpol[8].x1=r;
	HCpol[8].x2=l;
	HCpol[8].x3=b;

}//Fin Calc_Pluton


/***************************************************************************
			Kepler_eq
	ENTREE: Eccentricit e, anomalie moyenne M.
	RETOUR: Anomalie eccentrique E.
**************************************************************************/
double Kepler_eq(double e, double M)
{
	double precision,u,u_p=0;
	int ct = 0;
	u = M;
	precision = 0.0000000000001;
	while (fabs(u - u_p) > precision)
	{
		  u_p = u;
		  u = M + e * sin(u);
		  ct++;
		  if (ct > 25)
		  {
			 precision = precision * 2;
			 ct = 0;
		  };
	};
	return u;
}//Fin Kepler_eq


/***************************************************************************
			Mod360
	ENTREE: nombre quelconque.
	RETOUR: nombre compris entre 0 et 360.
**************************************************************************/
double Mod360(double nombre)
{
	while (nombre>360.0)
		nombre = nombre - 360.0;
	while (nombre<0.0)
		nombre = nombre + 360.0;

	return nombre;
}// fin Mod360


/***************************************************************************
			HCpol_to_GCpol
	Calcul des coordonnes gocentriques cliptiques polaires
	PRE: HCpol[i] et GCpol[2] calcules.
	POST: GCpol[i] calcules.
**************************************************************************/
void HCpol_to_GCpol(double T)
{
	int i;
	double N,D;

	for(i=0;i<9;i++)
		{
		if(i==2) i++;
		// N = r*cos(b)*sin(l-lsol)
		N = HCpol[i].x1 * cos(HCpol[i].x3*PIs180)
		* sin((HCpol[i].x2-GCpol[2].x2)*PIs180);
		// D = r*cos(b)*cos(l-lsol) + rsol
		D = HCpol[i].x1 * cos(HCpol[i].x3*PIs180)
		* cos((HCpol[i].x2-GCpol[2].x2)*PIs180) + HCpol[2].x1;
		//printf("\n%s\tN = %f\tD = %f",nom[i],N,D);

		GCpol[i].x1 = sqrt(N*N + D*D
				   + HCpol[i].x1*sin(HCpol[i].x3*PIs180)
				    *HCpol[i].x1*sin(HCpol[i].x3*PIs180));
		GCpol[i].x2 = atan2(N,D)*180.0/PI + GCpol[2].x2;
		GCpol[i].x2 = Mod360(GCpol[i].x2);
		GCpol[i].x3 = asin(HCpol[i].x1 / GCpol[i].x1
				   * sin(HCpol[i].x3*PIs180))*180.0/PI;
		}
}// fin HCpol_to_GCpol


/***************************************************************************
			Pol_to_cart
	Transforme des coordonnes polaires en coordonnes cartsiennes
	pol->x2 et pol->x3 doivent tre en degrs.
**************************************************************************/
Pol_to_cart(coord *pol, coord *cart)
{
	cart->x1 = pol->x1*cos(pol->x2*PIs180)*cos(pol->x3*PIs180);
	cart->x2 = pol->x1*cos(pol->x2*PIs180)*sin(pol->x3*PIs180);
	cart->x3 = pol->x1*sin(pol->x3*PIs180);
}// fin Pol_to_cart
